@@ -67,6 +67,10 @@ typedef struct {
 
     char * DateTimePointers[MAX_DATE_COPIES];
     int  numDateTimeTags;
+
+    int GpsInfoPresent;
+    char GpsLat[30];
+    char GpsLong[30];
 }ImageInfo_t;
 
 
@@ -96,10 +100,11 @@ void PrintFormatNumber(void * ValuePtr, int Format, int ByteCount);
 double ConvertAnyFormat(void * ValuePtr, int Format);
 int Get16u(void * Short);
 unsigned Get32u(void * Long);
+int Get32s(void * Long);
 
 //--------------------------------------------------------------------------
 // Exif format descriptor stuff
-extern int BytesPerFormat[];
+extern const int BytesPerFormat[];
 #define NUM_FORMATS 12
 
 #define FMT_BYTE       1 
@@ -116,12 +121,13 @@ extern int BytesPerFormat[];
 #define FMT_DOUBLE    12
 
 
-
-
 // makernote.c prototypes
 extern void ProcessMakerNote(unsigned char * DirStart, int ByteCount,
                  unsigned char * OffsetBase, unsigned ExifLength);
 
+// gpsinfo.c prototypes
+void ProcessGpsInfo(unsigned char * ValuePtr, int ByteCount, 
+                unsigned char * OffsetBase, unsigned ExifLength);
 
 
 // Prototypes for myglob.c module
@@ -170,3 +176,6 @@ extern int ShowTags;
 #define M_JFIF  0xE0            // Jfif marker
 #define M_EXIF  0xE1            // Exif marker
 #define M_COM   0xFE            // COMment 
+#define M_DQT   0xDB
+#define M_DHT   0xC4
+#define M_DRI   0xDD
