@@ -34,7 +34,7 @@
 ImageInfo_t ImageInfo;
 
 
-#define MAX_SECTIONS 100
+#define MAX_SECTIONS 40
 static Section_t Sections[MAX_SECTIONS];
 static int SectionsRead;
 static int HaveAll;
@@ -245,7 +245,7 @@ int ReadJpegSections (FILE * infile, ReadMode_t ReadMode)
                 // that uses marker 31 for non exif stuff.  Thus make sure 
                 // it says 'Exif' in the section before treating it as exif.
                 if ((ReadMode & READ_EXIF) && memcmp(Data+2, "Exif", 4) == 0){
-                    process_EXIF(Data, itemlen);
+                    process_EXIF((char *)Data, itemlen);
                 }else{
                     // Discard this section.
                     free(Sections[--SectionsRead].Data);
@@ -321,106 +321,29 @@ int ReadJpegFile(const char * FileName, ReadMode_t ReadMode)
     return ret;
 }
 
-
 //--------------------------------------------------------------------------
-// Replace or remove exif thumbnail
+// Remove exif thumbnail
 //--------------------------------------------------------------------------
-int SaveThumbnail(char * ThumbFileName)
+int TrimExifFunc(void)
 {
-    FILE * ThumbnailFile;
-
-    if (ImageInfo.ThumbnailOffset == 0 || ImageInfo.ThumbnailSize == 0){
-        printf("Image contains no thumbnail\n");
-        return FALSE;
-    }
-
-    if (strcmp(ThumbFileName, "-") == 0){
-        // A filename of '-' indicates thumbnail goes to stdout.
-        // This doesn't make much sense under Windows, so this feature is unix only.
-        ThumbnailFile = stdout;
-    }else{
-        ThumbnailFile = fopen(ThumbFileName,"wb");
-    }
-
-    if (ThumbnailFile){
-        uchar * ThumbnailPointer;
-        Section_t * ExifSection;
-        ExifSection = FindSection(M_EXIF);
-        ThumbnailPointer = ExifSection->Data+ImageInfo.ThumbnailOffset+8;
-
-        fwrite(ThumbnailPointer, ImageInfo.ThumbnailSize ,1, ThumbnailFile);
-        fclose(ThumbnailFile);
-        return TRUE;
-    }else{
-        ErrFatal("Could not write thumbnail file");
-        return FALSE;
-    }
-}
-
-//--------------------------------------------------------------------------
-// Replace or remove exif thumbnail
-//--------------------------------------------------------------------------
-int ReplaceThumbnail(const char * ThumbFileName)
-{
-    FILE * ThumbnailFile;
-    int ThumbLen, NewExifSize;
-    Section_t * ExifSection;
-    uchar * ThumbnailPointer;
-
-    if (ImageInfo.ThumbnailOffset == 0 || ImageInfo.ThumbnailAtEnd == FALSE){
-        // Adding or removing of thumbnail is not possible - that would require rearranging
-        // of the exif header, which is risky, and jhad doesn't know how to do.
-
-        printf("Image contains no thumbnail to replace - add is not possible\n");
-        return FALSE;
-    }
-
-    if (ThumbFileName){
-        ThumbnailFile = fopen(ThumbFileName,"rb");
-
-        if (ThumbnailFile == NULL){
-            ErrFatal("Could not read thumbnail file");
-            return FALSE;
+    int a;
+    for (a=0;a<SectionsRead-1;a++){
+        if (Sections[a].Type == M_EXIF && memcmp(Sections[a].Data+2, "Exif",4)==0){
+            unsigned int NewSize;
+            NewSize = RemoveThumbnail(Sections[a].Data, Sections[a].Size);
+            // Truncate the thumbnail section of the exif.
+            printf("%d bytes removed\n",Sections[a].Size-NewSize);
+            if (Sections[a].Size == NewSize) return FALSE; // Nothing removed.
+            Sections[a].Size = NewSize;
+            Sections[a].Data[0] = (uchar)(NewSize >> 8);
+            Sections[a].Data[1] = (uchar)NewSize;
+            return TRUE;
         }
-
-        // get length
-        fseek(ThumbnailFile, 0, SEEK_END);
-
-        ThumbLen = ftell(ThumbnailFile);
-        fseek(ThumbnailFile, 0, SEEK_SET);
-
-        if (ThumbLen + ImageInfo.ThumbnailOffset > 0x10000-20){
-            ErrFatal("Thumbnail is too large to insert into exif header");
-        }
-    }else{
-        ThumbLen = 0;
-        ThumbnailFile = NULL;
     }
-
-    ExifSection = FindSection(M_EXIF);
-
-    NewExifSize = ImageInfo.ThumbnailOffset+8+ThumbLen;
-    ExifSection->Data = (uchar *)realloc(ExifSection->Data, NewExifSize);
-
-    ThumbnailPointer = ExifSection->Data+ImageInfo.ThumbnailOffset+8;
-
-    if (ThumbnailFile){
-        fread(ThumbnailPointer, ThumbLen, 1, ThumbnailFile);
-        fclose(ThumbnailFile);
-    }
-
-    ImageInfo.ThumbnailSize = ThumbLen;
-
-    Put32u(ExifSection->Data+ImageInfo.ThumbnailSizeOffset+8, ThumbLen);
-
-    ExifSection->Data[0] = (uchar)(NewExifSize >> 8);
-    ExifSection->Data[1] = (uchar)NewExifSize;
-    ExifSection->Size = NewExifSize;
-
-    return TRUE;
+    // Not an exif image.  Can't remove exif thumbnail.
+    return FALSE;
 }
 
-
 //--------------------------------------------------------------------------
 // Discard everything but the exif and comment sections.
 //--------------------------------------------------------------------------
@@ -502,8 +425,7 @@ void WriteJpegFile(const char * FileName)
 Section_t * FindSection(int SectionType)
 {
     int a;
-
-    for (a=0;a<SectionsRead;a++){
+    for (a=0;a<SectionsRead-1;a++){
         if (Sections[a].Type == SectionType){
             return &Sections[a];
         }
